@@ -1046,9 +1046,16 @@ def fileOrImage(*args, **kwargs):
 
             # in-memory image - we have
             # to save it out to a file
-            if infile is None:
-                hd, infile = tempfile.mkstemp(fslimage.defaultExt())
+            if infile is None or not op.exists(infile):
+                hd, infile = tempfile.mkstemp(fslimage.defaultExt(),
+                                              dir=workdir)
                 os.close(hd)
+
+                # Create a copy of the input image and
+                # save that, so the original doesn't
+                # get associated with the temp file
+                val = nib.nifti1.Nifti1Image(
+                    np.asanyarray(val.dataobj), None, val.header)
                 val.to_filename(infile)
 
         return infile
@@ -1110,7 +1117,7 @@ def fileOrArray(*args, **kwargs):
         infile = None
 
         if isinstance(val, np.ndarray):
-            hd, infile = tempfile.mkstemp('.txt')
+            hd, infile = tempfile.mkstemp('.txt', dir=workdir)
             os.close(hd)
             np.savetxt(infile, val, fmt='%0.18f')
 
@@ -1176,6 +1183,7 @@ def fileOrText(*args, **kwargs):
         if not isinstance(val, pathlib.Path):
             with tempfile.NamedTemporaryFile(mode='w',
                                              suffix='.txt',
+                                             dir=workdir,
                                              delete=False) as f:
                 f.write(val)
                 infile = f.name
